@@ -23,6 +23,9 @@ import os
 from optparse import OptionParser
 import xmlrpclib
 import json
+import logging
+
+import stratuslab.api.LogUtil as LogUtil
 
 import stratuslab.Util as Util
 from stratuslab.VersionChecker import VersionChecker
@@ -30,14 +33,15 @@ import stratuslab.Exceptions as Exceptions
 from stratuslab.ConfigHolder import ConfigHolder, UserConfigurator
 from stratuslab.Exceptions import ConfigurationException
 
+
 class CommandBase(object):
-    
     def __init__(self):
         self.options = None
-        self.config  = {}
+        self.config = {}
         self._configKeysClassAttrsTwoWayMap = {}
 
         self.verboseLevel = 0
+        self.quiet = 0
         self.parser = None
         self._setParserAndParse()
 
@@ -49,17 +53,37 @@ class CommandBase(object):
         self.checkOptions()
         self._callAndHandleErrors(self, self.doWork.__name__)
 
+    def _getLoggingLevel(self):
+        if self.quiet >= 3:
+            return logging.CRITICAL
+        elif self.quiet == 2:
+            return logging.ERROR
+        elif self.quiet == 1:
+            return logging.WARNING
+        elif self.verboseLevel == 0:
+            return logging.INFO
+        else:
+            return logging.DEBUG
+
     def _setParserAndParse(self):
-        self.parser = OptionParser(version="2.9-SNAPSHOT")
+        self.parser = OptionParser(version="13.05.0")
+
         self.parser.add_option('-v', '--verbose', dest='verboseLevel',
-                help='verbose level. Add more to get more details',
-                action='count', default=self.verboseLevel)
+                               help='verbose level (use multiple times for more detail)',
+                               action='count', default=self.verboseLevel)
+
+        self.parser.add_option('-q', '--quiet', dest='quiet',
+                               help='quiet output (use multiple times for less detail)',
+                               action='count', default=self.quiet)
 
         self._addConfigFileOption()
 
         self.parse()
         self.verboseLevel = self.options.verboseLevel
-        
+        self.quiet = self.options.quiet
+
+        LogUtil.set_logger_level(level=self._getLoggingLevel())
+
     def _addConfigFileOption(self):
         pass
 
@@ -67,7 +91,7 @@ class CommandBase(object):
         pass
 
     def _callAndHandleErrors(self, methodName, *args, **kw):
-        
+
         try:
             Util.runMethodByName(methodName, *args, **kw)
         except ValueError, ex:
@@ -76,11 +100,10 @@ class CommandBase(object):
             self.raiseOrDisplayError('Error: %s' % ex.errmsg)
         except socket.sslerror, ex:
             self._checkPythonVersionAndRaise()
-            self.raiseOrDisplayError('Network error: %s' % ex)
+            raise ex
+            #self.raiseOrDisplayError('Network error: %s' % ex)
         except socket.error, ex:
             self.raiseOrDisplayError('Network error: %s' % ex)
-        except socket.gaierror, ex:
-            self.raiseOrDisplayError('Network error: %s' % ex)
         except Exceptions.ClientException, ex:
             msg = 'Error: ' + ex.reason
             if ex.content and (self.verboseLevel > 1):
@@ -97,7 +120,7 @@ class CommandBase(object):
             VersionChecker().check()
         except Exceptions.ValidationException, ex:
             self.raiseOrDisplayError(ex)
-        
+
     def parse(self):
         pass
 
@@ -115,7 +138,7 @@ class CommandBase(object):
 
     def usageExitWrongNumberOfArguments(self):
         return self.parser.error('Wrong number of arguments')
-    
+
     def raiseOrDisplayError(self, errorMsg):
         if self.verboseLevel > 2:
             raise
@@ -128,12 +151,12 @@ class CommandBase(object):
 
     def printError(self, message):
         Util.printError(message, self)
-        
+
     def printMandatoryOptionError(self, option):
         self.printError('Missing mandatory %s option' % option)
 
-class CommandBaseUser(CommandBase):
 
+class CommandBaseUser(CommandBase):
     def _addConfigFileOption(self):
         ConfigHolder.addConfigFileUserOption(self.parser)
 
@@ -149,8 +172,8 @@ class CommandBaseUser(CommandBase):
 
         if configFile == Util.defaultConfigFileUser:
             if not os.path.exists(configFile):
-                Util.printDetail('[WARNING] Default user configuration file does not exist: %s' % 
-                                  configFile, verboseLevel=self.verboseLevel)
+                Util.printDetail('[WARNING] Default user configuration file does not exist: %s' %
+                                 configFile, verboseLevel=self.verboseLevel)
                 return
 
         selected_section = None
@@ -159,18 +182,20 @@ class CommandBaseUser(CommandBase):
 
         try:
             self.config, self._configKeysClassAttrsTwoWayMap = \
-                UserConfigurator.configFileToDictWithFormattedKeys(configFile, withMap=True, selected_section=selected_section)
+                UserConfigurator.configFileToDictWithFormattedKeys(configFile, withMap=True,
+                                                                   selected_section=selected_section)
         except ConfigurationException, ex:
             raise ConfigurationException('Error parsing user configuration file %s' % configFile + '. Details: %s' % ex)
 
     def _updateOptionsFromConfigFile(self):
-        """Order of precedence:
+        """
+        Order of precedence:
         * command line option
         * environment variable
         * configuration file
         * default value
 
-        Update of the corresponding options object key/value pairs can only be 
+        Update of the corresponding options object key/value pairs can only be
         done if neither corresponding command line option nor environment
         variable was given/set.
 
@@ -187,20 +212,20 @@ class CommandBaseUser(CommandBase):
         for k in self.config:
             if k in self.options.__dict__:
                 valueFromOptions = getattr(self.options, k)
-                # Set the value from configuration file: 
+                # Set the value from configuration file:
                 # * if attribute is empty
                 if not valueFromOptions:
                     setattr(self.options, k, self.config[k])
-                # * if default is equal to "provided" value, we may assume that 
-                #   the option wasn't given on command line; but we are going 
-                #   to double check this in each Option object of the parser as 
+                # * if default is equal to "provided" value, we may assume that
+                #   the option wasn't given on command line; but we are going
+                #   to double check this in each Option object of the parser as
                 #   the same value might have been provided via environment var.
                 elif valueFromOptions == self.parser.defaults[k]:
                     for optionObj in self.parser.option_list:
-                        # work only with Option object for the particular key 
+                        # work only with Option object for the particular key
                         if optionObj.dest != k:
                             continue
-                        # Update if
+                            # Update if
                         # * the long option is NOT in the list of the CLI arguments
                         if not (optionObj._long_opts[0] in sys.argv):
                             # * and not set via corresponding environment variable
@@ -210,7 +235,7 @@ class CommandBaseUser(CommandBase):
                 else:
                     pass
 
-class CommandBaseSysadmin(CommandBase):
 
+class CommandBaseSysadmin(CommandBase):
     def _addConfigFileOption(self):
         ConfigHolder.addConfigFileSysadminOption(self.parser)
